##############################################################################
# Makefile for all MPI Fortran example codes
# LAST REVISED:  12/27/2001 Blaise Barney
###############################################################################

F77     =   mpxlf
CC	=   cc
FLAGS   =   -O2

all:    mpi_2dfft       \
        mpi_array       \
        mpi_bandwidth   \
        mpi_cartesian   \
        mpi_contig      \
        mpi_group       \
        mpi_heat2D      \
        mpi_indexed     \
        mpi_mm          \
        mpi_pi_reduce   \
        mpi_pi_send     \
        mpi_ping        \
        mpi_prime       \
        mpi_ringtopo    \
        mpi_scatter     \
        mpi_struct      \
        mpi_latency     \
        mpi_wave        \
        mpi_vector

clean:  
	/bin/rm -ef     \
	initial.dat     \
	final.dat       \
	mpi_2dfft       \
	mpi_array       \
	mpi_bandwidth   \
	mpi_cartesian   \
	mpi_contig      \
	mpi_group       \
	mpi_heat2D      \
	mpi_indexed     \
	mpi_mm          \
	mpi_pi_reduce   \
	mpi_pi_send     \
	mpi_ping        \
	mpi_prime       \
	mpi_ringtopo    \
	mpi_scatter     \
	mpi_struct      \
	mpi_latency     \
	mpi_wave        \
	mpi_vector      \
        *.o


draw_heat.o:  draw_heat.c
	$(CC) -c $(FLAGS) draw_heat.c 

draw_wave.o:  draw_wave.c
	$(CC) -c draw_wave.c 

mpi_2dfft:  mpi_2dfft.f ser_fft.o timing_fgettod.o
	$(F77) $(FLAGS) mpi_2dfft.f ser_fft.o timing_fgettod.o -o mpi_2dfft

mpi_array:  mpi_array.f
	$(F77) $(FLAGS) mpi_array.f -o mpi_array

mpi_bandwidth:  mpi_bandwidth.f 
	$(F77) $(FLAGS) mpi_bandwidth.f -o mpi_bandwidth

mpi_cartesian:  mpi_cartesian.f 
	$(F77) $(FLAGS) mpi_cartesian.f -o mpi_cartesian

mpi_contig:  mpi_contig.f 
	$(F77) $(FLAGS) mpi_contig.f -o mpi_contig

mpi_group:  mpi_group.f 
	$(F77) $(FLAGS) mpi_group.f -o mpi_group

mpi_heat2D:  mpi_heat2D.f draw_heat.o
	$(F77) $(FLAGS) mpi_heat2D.f draw_heat.o -o mpi_heat2D -lX11

mpi_indexed:  mpi_indexed.f 
	$(F77) $(FLAGS) mpi_indexed.f -o mpi_indexed

mpi_mm:  mpi_mm.f
	$(F77) $(FLAGS) mpi_mm.f -o mpi_mm

mpi_pi_reduce:  mpi_pi_reduce.f dboard.f
	$(F77) $(FLAGS) mpi_pi_reduce.f dboard.f -o mpi_pi_reduce

mpi_pi_send:  mpi_pi_send.f dboard.f
	$(F77) $(FLAGS) mpi_pi_send.f dboard.f -o mpi_pi_send

mpi_prime:  mpi_prime.f 
	$(F77) $(FLAGS) mpi_prime.f -o mpi_prime

mpi_ping:  mpi_ping.f 
	$(F77) $(FLAGS) mpi_ping.f -o mpi_ping

mpi_ringtopo:  mpi_ringtopo.f 
	$(F77) $(FLAGS) mpi_ringtopo.f -o mpi_ringtopo

mpi_scatter:  mpi_scatter.f 
	$(F77) $(FLAGS) mpi_scatter.f -o mpi_scatter

mpi_struct:  mpi_struct.f 
	$(F77) $(FLAGS) mpi_struct.f -o mpi_struct

mpi_latency:  mpi_latency.f 
	$(F77) $(FLAGS) mpi_latency.f -o mpi_latency

mpi_wave:  mpi_wave.f draw_wave.o
	$(F77) $(FLAGS) mpi_wave.f draw_wave.o -o mpi_wave -lX11

mpi_vector:  mpi_vector.f 
	$(F77) $(FLAGS) mpi_vector.f -o mpi_vector

ser_fft.o: ser_fft.f
	$(F77) $(FLAGS) -c ser_fft.f

timing_fgettod.o:  timing_fgettod.c
	$(CC) -c $(FLAGS) timing_fgettod.c 

