/****************************************************************************
* FILE: allreduce.c
* DESCRIPTION:
*  This example code uses the MPI_Allreduce() routine to demonstrate 
*  performance differences between polling and interrupt modes.  
*  On the IBM SP, the default mode is polling.  This can be overridden by
*  setting the MP_CSS_INTERRUPT environment variable to "yes".  The 
*  MP_EUILIB environment variable can be set to "us" for User Space
*  communications, or to "ip" for Internet Protocol.
* LAST REVISED: 01/09/99 Blaise Barney
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define REPS 10000
#define TRIALS 10

int main(argc, argv)
int argc;
char **argv;
{

int     rank, size, i, j;
double  t1, t2, in=1.0, out, ttime=0.0;

MPI_Init(&argc, &argv);
MPI_Comm_rank(MPI_COMM_WORLD, &rank);
MPI_Comm_size(MPI_COMM_WORLD, &size);

if (rank == 0) {
  printf("\n******************** MPI_Allreduce *********************\n");
  printf("Using %d tasks.  MP_CSS_INTERRUPT = %s\n",size, 
         getenv("MP_CSS_INTERRUPT"));
  printf("Doing %d trials of %d reps with message size= %d bytes\n\n",TRIALS,
         REPS, sizeof(in));

  for (j=0; j<TRIALS; j++) {
    MPI_Barrier(MPI_COMM_WORLD);   /* synchronize all tasks */

    t1 = MPI_Wtime();
    for (i=0; i<REPS; i++) 
      MPI_Allreduce(&in, &out, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
    t2 = MPI_Wtime();

    ttime = ttime + (t2-t1);
    printf("Trial %d: time= %f\n", j, (t2-t1)/REPS);
    }
  printf("\nAverage time= %f seconds\n", ttime/(REPS*TRIALS));
  }

else {
  for (j=0; j<TRIALS; j++) {
    MPI_Barrier(MPI_COMM_WORLD);   /* synchronize all tasks */
    for (i=0; i<REPS; i++)
      MPI_Allreduce(&in, &out, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
    }
  }

MPI_Finalize();
}
