C----------------------------------------------------------------------------- 
C FILE: gettime.f
C DESCRIPTION: Demonstrates use of the gettimeofday function as it would be
C   called from a Fortran program.
C LAST REVISED: 01/09/96 Blaise Barney
C----------------------------------------------------------------------------- 
C Because Fortran must make an interlanguage call to gettimeofday, a small
C C "wrapper" program (below) must first be compiled to produce an object 
C file called timing_fgettod.o. For example:  cc -c timing_fgettod.c
C 
C    #include <stdio.h>
C    #include <sys/time.h>
C    void timing_fgettod (int times[2]) 
C    {
C      struct timeval tv;
C      gettimeofday(&tv, (struct timeval*)0);
C      times[0] = tv.tv_sec;
C      times[1] = tv.tv_usec;
C    }
C 
C Now, the Fortran code which makes the interlanguage call to timing_fgettod.
C Note that with xlf, this Fortran code is compiled with the C object file
C timing_fgettod.o to produce the executable.  For example, if the Fortran
C source code below was called gettime.f, then the compile command would
C be:   xlf -o gettime gettime.f timing_fgettod.o

      program gettime
      external timing_fgettod
      integer ARRAY_SIZE, SEED
      parameter(ARRAY_SIZE=1000)
      parameter(SEED=1995)

      real*8 a(ARRAY_SIZE,ARRAY_SIZE)
      real*8 b(ARRAY_SIZE,ARRAY_SIZE)
      real*8 c(ARRAY_SIZE,ARRAY_SIZE)
      integer times1(2), times2(2)
      integer i, j, total_usecs

C     First, call timing_fgettod to get the start time
      call timing_fgettod(%REF(times1))

      do i=1,1000
        do j=1,1000
          c(i,j) = c(i,j) + a(i,j) * b(i,j)
        end do
      end do

C     Now call timing_fgettod to get the end time
      call timing_fgettod(%REF(times2))

C     Print the execution time
      total_usecs = ((times2(1) - times1(1))*1000000) + times2(2) -
     &              times1(2)
      print *, 'Total time was ',total_usecs, 'uSec'
      end
