/*----------------------------------------------------------------*/
/* NOTE: this version times a simple integer-intensive loop to    */
/* "measure" the clock frequency. This works on power-2, power-pc,*/
/* and power-3 architectures; but the code MUST be compiled with  */
/* "cc qcpu.c -o qcpu"  without optimization or debugging.        */
/* Use of any other compiler flags might give wrong answers for   */
/* the apparent clock frequency, and the apparent frequency might */
/* be wrong for other chip architectures.                         */
/*----------------------------------------------------------------*/

#include <sys/time.h>
#include <sys/types.h>
#include <sys/systemcfg.h>
#include <stdio.h>
double walltime(void);

 void main()

 {
   int fd;
   int nprocs, imp;
   int i, isum;
   int iter, maxiter;
   double freq, maxfreq;
   double time1, time2;
   double fudge;

   nprocs = _system_configuration.ncpus;
   printf("number of cpus: %d\n", nprocs);
   imp = _system_configuration.implementation;
   if (imp == POWER_RS1)
      printf("processor class: POWER_RS1\n");
   else if (imp == POWER_RSC)
      printf("processor class: POWER_RSC\n");
   else if (imp == POWER_RS2)
      printf("processor class: POWER_RS2\n");
   else if (imp == POWER_601)
      printf("processor class: POWER_601\n");
   else if (imp == POWER_603)
      printf("processor class: POWER_603\n");
   else if (imp == POWER_604)
      printf("processor class: POWER_604\n");
   else if (imp == POWER_630)
      printf("processor class: POWER_630\n");
   else if (imp == POWER_RS64II)
      printf("processor class: POWER_RS64II\n");
   else
   {
      printf("processor class: unknown\n");
      printf("warning: the reported frequency may be incorrect\n");
   }

   if (imp == POWER_RS64II)
     fudge = 1.0;
   else
     fudge = 0.90;

   printf("L1 Inst Cache Size  in KB: %d\n",_system_configuration.icache_size /
1024);
   printf("L1 Data Cache Size  in KB: %d\n",_system_configuration.dcache_size /
1024);
   printf("L1 Data Cache Line Sz (B): %d\n",_system_configuration.dcache_line );
   printf("L2 Cache Size       in KB: %d\n",_system_configuration.L2_cache_size
/ 1024);
   printf("data TLB: Size = %d, associativity = %d\n",
          _system_configuration.dtlb_size, _system_configuration.dtlb_asc);
   printf("instruction TLB: Size = %d, associativity = %d\n",
          _system_configuration.itlb_size, _system_configuration.itlb_asc);


   maxiter = 40;

   maxfreq = 0.0;
   for (iter=0; iter<maxiter; iter++)
   {
      time1 = walltime();
      isum = 0;
      for (i=0; i<100000; i++) isum += 1;
      time2 = walltime();
      freq = fudge/(time2-time1);
      if (freq > maxfreq) maxfreq = freq;
   }

   printf("apparent freq = %.1f MHz\n", maxfreq );

}

/*-------------------------------------------------*/
/* wall clock time in seconds from real-time-clock */
/*-------------------------------------------------*/
double walltime(void)
{
    struct timebasestruct TB;
    double wtime;

    read_real_time(&TB, TIMEBASE_SZ);
    time_base_to_time(&TB, TIMEBASE_SZ);

    wtime =            (double) TB.tb_high
          + 1.0e-9 * ( (double) TB.tb_low );

    return wtime;
}
