#include <stdio.h>

#define abs(x) ((x) >= 0 ? (x) : -(x))
#define max(a,b) ((a) >= (b) ? (a) : (b))

/*******************************************************************/
/*     MESHGENERATION WITH THOMPSONS SOLVER, VECTORIZED VERSION    */
/*     SET  N = 33, 65, 129                                        */
/*     DO  SCALAR AND VECTOR  RUNS                                 */
/*     W.GENTZSCH   FH REGENSBURG   F.R.G.                         */
/*******************************************************************/

void main() {

    /* Local variables */
    static long ixcm, lmax, jxcm, iycm, jycm, irxm, jrxm, irym, jrym;
    static long i, j;
    static long ll;

    static double alfa, dmax__, dxcm, dycm;
    static double relfa;
    static double abx, aby, rel, eps, rxm, rym;
    static double d[257][257], 
                  x[257][257], 
		  y[257][257], 
		  aa[257][257], 
		  dd[257][257];
    static double rx[257][257], 
		  ry[257][257];

    alfa = .1;
    relfa = .98;
    lmax = 100;
    eps = 5e-8;
    rel = 2. / relfa;

    /* INITIAL FIELDS FOR X(I,J) AND Y(I,J) WITH STRECH FACTOR ALFA */

    for (i = 1; i <= 257; ++i) {
	x[i-1][0] = (i - 1) / 256.;
    }

    for (i = 1; i <= 257; ++i) {
	x[i-1][256] = x[i-1][0];
	x[0][i-1] = 0.;
	x[256][i-1] = 1.;
	y[i-1][0] = 0.;
	y[0][i-1] = x[i-1][0];
	y[256][i-1] = alfa * x[i-1][0];
    }
    for (i = 1; i <= 257; ++i) {
	y[i-1][256] = (1. - x[i-1][0]) * y[0][256] + x[i-1][0] * y[256][256];
    }
    #pragma omp parallel for \
		     private (i,j) \
		     shared  (x,y)
    for (i = 2; i <= 256; ++i) {
	for (j = 2; j <= 256; ++j) {
	    x[i-1][j-1] = x[i-1][0] * .9;
	    y[i-1][j-1] = ((1. - x[i-1][0]) * y[0][j-1] + x[i-1][0] * y[256][j-1]) * .9;
	}
    }

    printf( "1  *** 2-D ITERATION BEHAVIOR ***\n" );
    printf( " IT   I   J   X-COR      I   J   Y-COR      I   J   X-RES      I   J   Y-RES\n" );




    ll = 0;
    /* START ITERATION LOOP 190 */
L190:

    ixcm = 0;
    jxcm = 0;
    dxcm = 0.;
    iycm = 0;
    jycm = 0;
    dycm = 0.;
    irxm = 0;
    jrxm = 0;
    rxm = 0.;
    irym = 0;
    jrym = 0;
    rym = 0.;

    #pragma omp parallel shared  (x,y,aa,dd,rel,rx,ry,rxm,irxm,jrxm,rym,irym,jrym,d)
    {
    long i,j;

    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	long ip = i + 1;
	long im = i - 1;

	for (j = 2; j <= 256; ++j) {
	    long m     = j - 1;
	    long jp    = j + 1;
	    long jm    = j - 1;
	    double xx  = x[ip-1][j-1] - x[im-1][j-1];
	    double yx  = y[ip-1][j-1] - y[im-1][j-1];
	    double xy  = x[i-1][jp-1] - x[i-1][jm-1];
	    double yy  = y[i-1][jp-1] - y[i-1][jm-1];
	    double a   = (xy * xy + yy * yy) * .25;
	    double b   = (xx * xx + yx * yx) * .25;
	    double c   = (xx * xy + yx * yy) * .125;
	    double qi  = 0.;
	    double qj  = 0.;
	    double pxx = x[ip-1][j-1]  - x[i-1][j-1]   *  2.            + x[im-1][j-1];
	    double qxx = y[ip-1][j-1]  - y[i-1][j-1]   *  2.            + y[im-1][j-1];
	    double pyy = x[i-1][jp-1]  - x[i-1][j-1]   *  2.            + x[i-1][jm-1];
	    double qyy = y[i-1][jp-1]  - y[i-1][j-1]   *  2.            + y[i-1][jm-1];
	    double pxy = x[ip-1][jp-1] - x[ip-1][jm-1] - x[im-1][jp-1] + x[im-1][jm-1];
	    double qxy = y[ip-1][jp-1] - y[ip-1][jm-1] - y[im-1][jp-1] + y[im-1][jm-1];

	    aa[i-1][m-1] = -b;
	    dd[i-1][m-1] = b + b + a * rel;

	    /* CALCULATE RESIDUALS ( EQUIVALENT TO RIGHT HAND SIDES OF EQUS.) */

	    rx[i-1][m-1] = a * pxx + b * pyy - c * pxy + xx * qi + xy * qj;
	    ry[i-1][m-1] = a * qxx + b * qyy - c * qxy + yx * qi + yy * qj;
	}
    }

    /* DETERMINE MAXIMUM VALUES OF RESIDUALS */

    #pragma omp sections
    {
	#pragma omp section
	for (i = 2; i <= 256; ++i)
	    for (j = 1; j <= 255; ++j) {
		double d__1;
		if ((d__1 = rx[i-1][j-1], abs(d__1)) >= abs(rxm)) {
		    rxm = rx[i-1][j-1];
		    irxm = i;
		    jrxm = j;
		}
	    }
	#pragma omp section
	for (i = 2; i <= 256; ++i)
	    for (j = 1; j <= 255; ++j) {
		double d__1;
		if ((d__1 = ry[i-1][j-1], abs(d__1)) >= abs(rym)) {
		    rym = ry[i-1][j-1];
		    irym = i;
		    jrym = j;
		}
	    }
    } /*end sections*/

    /* SOLVE TRIDIAGONAL SYSTEMS IN PARALLEL */

    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	d[i-1][0] = 1. / dd[i-1][0];
    }
    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	for (j = 2; j <= 255; ++j) {
	    double r           = aa[i-1][j-1] * d[i-1][j-2];
	    d[i-1][j-1]   = 1. / (dd[i-1][j-1] - aa[i-1][j-2] * r);
	    rx[i-1][j-1] -= rx[i-1][j-2] * r;
	    ry[i-1][j-1] -= ry[i-1][j-2] * r;
	}
    }
    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	rx[i-1][254] *= d[i-1][254];
	ry[i-1][254] *= d[i-1][254];
    }
    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	for (j = 2; j <= 255; ++j) {
	    long k = 255 - j + 1;
	    rx[i-1][k-1] = (rx[i-1][k-1] - aa[i-1][k-1] * rx[i-1][k]) * d[i-1][k-1];
	    ry[i-1][k-1] = (ry[i-1][k-1] - aa[i-1][k-1] * ry[i-1][k]) * d[i-1][k-1];
	}
    }

    /* ADD CORRECTIONS */

    #pragma omp for
    for (i = 2; i <= 256; ++i) {
	for (j = 2; j <= 256; ++j) {
	    long l = j - 1;
	    x[i-1][j-1] += rx[i-1][l-1];
	    y[i-1][j-1] += ry[i-1][l-1];
	}
    }

    } /*end parallel*/

    /* PREPARE OUTPUT OF CONVERGENCE BEHAVIOUR */

    ++ll;
    printf("%3d%4d%4d%11.4E%4d%4d%11.4E%4d%4d%11.4E%4d%4d%11.4E\n",
	ll, ixcm, jxcm, dxcm, iycm, jycm, dycm, irxm, jrxm, rxm, irym, jrym, rym );

    abx = abs(rxm);
    aby = abs(rym);
    dmax__ = max(abx,aby);
    if (ll < lmax && dmax__ > eps) {
	/* END OF ITERATION LOOP 190 */
	goto L190;
    }


}

