#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <omp.h>

#include <math.h>
#include <time.h>
#include <assert.h>

#define TRUE	1
#define logi(x) 	(log((double)(x)))
#define min(x,y)	(((x) < (y)) ? (x) : (y))
#define max(x,y)	(((x) > (y)) ? (x) : (y))

#define LEAF_TASKS_PER_THREAD 16 
#define ARRAY_SIZE 1000000
#define REPETITIONS 10
#define SAMPLE_MIN 5
#define SAMPLE_RATIO 64

long A[ARRAY_SIZE];

long partition(long A[], long p, long r, long n)
{
    long x, i, j, t;

    n = min(n, r-p+1);
    switch (n) {

      case 1:
	x = A[p];
	break;

      case 2:
        x = (A[p] + A[r]) / 2;
	break;

      case 3:
        x = (A[p] + A[p+((r-p)/2)] + A[r]) / 3;
	break;

      default:
	{
	    long s = (r-p)/(n-1);
	    x = 0;
	    for (i=0; i<n; i++) x += A[p+i*s];
	    x /= n;
	}
	break;
    }

    i = p - 1;
    j = r + 1;

    while (TRUE) {
	while (A[--j] > x);
	while (A[++i] < x);

	if (i < j) {
	    assert(j >= p && j <= r);
	    assert(i >= p && i <= r);
	    t = A[j];
	    A[j] = A[i];
	    A[i] = t;
	} else {
	    assert(j >= p && j < r);
	    return j;
	}
    }
}


void quicksort(long A[], long p, long r, long d, long qdepth)
{
    if (p < r) {

	if (d <= qdepth) {

#ifdef WAIT
	    #pragma omp taskq 
#else
	    #pragma omp taskq nowait firstprivate(A, p, r, d, qdepth)
#endif
	    {
	        long q, n;
		n = (r-p)/SAMPLE_RATIO;
		q = partition(A, p, r, max(SAMPLE_MIN, n));

		#pragma omp task
                quicksort(A, p, q, d+1, qdepth);

		#pragma omp task
                quicksort(A, q+1, r, d+1, qdepth);
	    }
	} else {
	    long n = (r-p)/SAMPLE_RATIO;
	    long q = partition(A, p, r, max(SAMPLE_MIN, n));
	    quicksort(A, p, q, d+1, qdepth);
	    quicksort(A, q+1, r, d+1, qdepth);
	}
    }
}


void 
main()
{ 
    long i, qdepth, needdepth, fulldepth, rep;
    double start, end, total = 0.0;

    srand((unsigned int)time(NULL));

    needdepth = logi(LEAF_TASKS_PER_THREAD * omp_get_max_threads()) / logi(2);
    fulldepth = logi(ARRAY_SIZE) / logi(2);
    qdepth = min(needdepth, fulldepth);

    for (rep = 1; rep <= REPETITIONS; rep++) {

	for (i = 0; i < ARRAY_SIZE; i++) {
	    A[i] = rand()*RAND_MAX + rand();
	}

	#pragma omp parallel default(none) shared(A, qdepth)
	    quicksort(A, 0, ARRAY_SIZE - 1, 1, qdepth);
	total += end - start;

	for (i = 1; i < ARRAY_SIZE; i++) assert(A[i-1] <= A[i]);
    }

    printf(" Sort successful!\n");
    exit(0);
}
